# accim - Adaptive-Comfort-Control-Implemented Model
# Copyright (C) 2021-2025 Daniel Sánchez-García

# accim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# accim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""
Run the function below to add the ACCIS.

This function transform fixed setpoint temperature
building energy models into adaptive setpoint temperature energy models
by adding the Adaptive Comfort Control Implementation Script (ACCIS)
"""
import pandas as pd
from accim import __version__
from accim import lists


class addAccis:
    """
    Adds the Adaptive-Comfort-Control Implementation Script, which is an EnergyManagementSystem
    script that applies adaptive setpoint temperatures to EnergyPlus building energy models.

    :param ScriptType: The default is None.
        'vrf_ac' for VRF system with full air-conditioning mode,
        'vrf_mm' for VRF system with mixed-mode,
        'ex_ac' for existing HVAC only with full air-conditioning mode,
        'ex_mm' for existing HVAC with mixed-mode.
    :type ScriptType: str
    :param SupplyAirTempInputMethod: The default is None.
        'supply air temperature' or 'temperature difference' to use such
        supply air temperature input method in the VRF system.
        Only used if vrf_ac or vrf_mm are used.
    :type SupplyAirTempInputMethod: str
    :param Output_type: The default is None.
        Can be 'standard', 'simplified', 'detailed' or 'custom'.
    :type Output_type: str
    :param Output_freqs: The default is None.
         A list containing the following strings:
         ['timestep', 'hourly', 'daily', 'monthly', 'runperiod']
    :type Output_freqs: list
    :param Output_keep_existing: The default is None.
        It is a boolean (True or False) to keep the existing Output:Variable objects or not.
    :type Output_keep_existing: bool
    :param Output_gen_dataframe: The default is None.
        It is a boolean (True or False) to generate a pandas DataFrame instance
        containing all Output:Variable objects.
    :type Output_gen_dataframe: bool
    :param Output_take_dataframe: It takes the pandas DataFrame previously generated
        with Output_gen_dataframe, which the user has filtered to keep only the rows
        related to the Output:Variable objects that need to be kept in the model.
    :type Output_take_dataframe: bool
    :param EnergyPlus_version: The default is 'auto'.
        Can be '9.1', '9.2', '9.3', '9.4', '9.5', '9.6', '22.1', '22.2', '23.1', '23.2', '24.1', '24.2', '25.1' or 'auto'.
    :type EnergyPlus_version: str
    :param TempCtrl: The default is None. Can be 'temp' or 'pmv'.
    :type TempCtrl: str
    :param ComfStand: The default is None.
        '0 = ESP CTE;
        '1 = INT EN16798;
        '2 = INT ASHRAE55;
        '3 = JPN Rijal;
        '4 = CHN GBT50785 Cold;
        '5 = CHN GBT50785 HotMild;
        '6 = CHN Yang;
        '7 = IND IMAC C NV;
        '8 = IND IMAC C MM;
        '9 = IND IMAC R 7DRM;
        '10 = IND IMAC R 30DRM;
        '11 = IND Dhaka;
        '12 = ROM Udrea;
        '13 = AUS Williamson;
        '14 = AUS DeDear;
        '15 = BRA Rupp NV;
        '16 = BRA Rupp AC;
        '17 = MEX Oropeza Arid;
        '18 = MEX Oropeza DryTropic;
        '19 = MEX Oropeza Temperate;
        '20 = MEX Oropeza HumTropic;
        '21 = CHL Perez-Fargallo;
        '22 = INT ISO7730;
        '99 = CUSTOM;
    :type ComfStand: list
    :param CustAST_m: The m coefficient (slope) of custom model linear regression (mx+n)
    :type CustAST_m: float
    :param CustAST_n: The n coefficient of custom model linear regression (mx+n)
    :type CustAST_n: float
    :param CustAST_AHSToffset: The offset for heating setpoint from neutral temperature
        for the custom model linear regression. This value will be summed, therefore, it must be negative.
    :type CustAST_AHSToffset: float
    :param CustAST_ACSToffset: The offset for cooling setpoint from neutral temperature
        for the custom model linear regression. This value will be summed, therefore, it must be positive.
    :type CustAST_ACSToffset: float
    :param CustAST_ACSTaul: The value for the cooling setpoint applicability upper limit (ACSTaul).
    :type CustAST_ACSTaul: float
    :param CustAST_ACSTall: The value for the cooling setpoint applicability lower limit (ACSTall).
    :type CustAST_ACSTall: float
    :param CustAST_AHSTaul: The value for the heating setpoint applicability upper limit (AHSTaul).
    :type CustAST_AHSTaul: float
    :param CustAST_AHSTall: The value for the heating setpoint applicability lower limit (AHSTall).
    :type CustAST_AHSTall: float
    :param CAT: The default is None.
        (1 = CAT I; 2 = CAT II; 3 = CAT III; 80 = 80% ACCEPT; 85 = 85% ACCEPT; 90 = 90% ACCEPT)
    :type CAT: list
    :param CATcoolOffset: An offset to modify comfort models.
        This value is summed to the predefined cooling setpoint offset for the CAT value.
    :type CATcoolOffset: float
    :param CATheatOffset: An offset to modify comfort models.
        This value is summed to the predefined heating setpoint offset for the CAT value.
    :type CATheatOffset: float
    :param ComfMod: The default is None.
        (0/0.X = Static;
        1/1.X = Adaptive when applicable, otherwise relevant local static model;
        2 = Adaptive when applicable, otherwise relevant international static model
        3 = Adaptive when applicable, otherwise horizontal extention of adaptive setpoints)
    :type ComfMod: list
    :param SetpointAcc: A float. It is the number for the accuracy of the setpoint temperatures.
        For instance, if 2 was used, setpoints would be rounded to every half Celsius degree;
        if 10 was used, the setpoints would be rounded to the first decimal.
    :type SetpointAcc: float
    :param CoolSeasonStart: A date in format dd/mm, or the number of the day in the year.
        Defines when start the cooling season, only used in some static setpoint temperatures.
    :type CoolSeasonStart: any
    :param CoolSeasonEnd: A date in format dd/mm, or the number of the day in the year.
        Defines when ends the cooling season, only used in some static setpoint temperatures.
    :type CoolSeasonEnd: any
    :param HVACmode: The default is None.
        (0 = Fully Air-conditioned; 1 = Naturally ventilated; 2 = Mixed Mode)
    :type HVACmode: list
    :param VentCtrl: The default is None.
        (if HVACmode = 1:
        0 = Ventilates above neutral temperature;
        1 = Ventilates above upper comfort limit;
        if HVACmode = 2:
        0 = Ventilates above neutral temperature and fully opens doors and windows;
        1 = Ventilates above lower comfort limit and fully opens doors and windows;
        2 = Ventilates above neutral temperature and opens doors and windows based on the customised venting opening factor;
        3 = Ventilates above lower comfort limit and opens doors and windows based on the customised venting opening factor;
        )
    :type VentCtrl: list
    :param MaxTempDiffVOF: The maximum temperature difference for the Venting Opening Factor.
        Must be a number greater than 0.
    :type MaxTempDiffVOF: float
    :param MinTempDiffVOF: The minimum temperature difference for the Venting Opening Factor.
        Must be a number greater than 0 and smaller than the maximum temperature difference.
    :type MinTempDiffVOF: float
    :param MultiplierVOF: The multiplier for the modulation of the Venting Opening Factor.
        Must be a number between 0 and 1.
    :type MultiplierVOF: float
    :param VSToffset: The default is 0. Please refer to documentation.
    :type VSToffset: list
    :param MinOToffset: The default is 50. Please refer to documentation.
    :type MinOToffset: list
    :param MaxWindSpeed: The default is 50. Please refer to documentation.
    :type MaxWindSpeed: list
    :param ASTtol_start: The default is 0.1. Please refer to documentation.
    :type ASTtol_start: float
    :param ASTtol_end_input: The default is 0.1. Please refer to documentation.
    :type ASTtol_end_input: float
    :param ASTtol_steps: The default is 0.1. Please refer to documentation.
    :type ASTtol_steps: float
    :param NameSuffix: The default is '' (an empty string). Please refer to documentation.
    :type NameSuffix: str
    :param eer: The energy efficiency ratio of the VRF system for each zone,
    added when using ScriptType vrf_mm or vrf_ac
    :type: eer: int
    :param cop: The coefficient of performance of the VRF system for each zone,
    added when using ScriptType vrf_mm or vrf_ac
    :type: cop: int
    :param make_averages: Used to make averages of hour-counting variables.
    :type make_averages: bool
    :param debugging: If True, an Output:EnergyManagementSystem object is used
        to generate the EDD file.
    :type debugging: bool
    :param verboseMode: True to print the process on screen. Default is True.
    :type verboseMode: bool
    :param confirmGen: True to skip confirmation of output IDF generation. Default is None.
    :type confirmGen: bool
    :ivar arguments: A dictionary containing all arguments
    :ivar df_outputs: the pandas DataFrame instance created with argument ``Output_gen_dataframe``
    :ivar input_idfs: A dictionary containing all input IDFs following the format {'input idf filename': class ``eppy.modeleditor.IDF object``}
    :ivar occupied_zones: A dictionary containing all input idfs and
        occupied zone names following the format {'idf filename': [list of zone names]}
    :ivar occupied_zones_original_name: A dictionary containing all input idfs and
        occupied zone original names following the format {'idf filename': [list of zone original names]}
    :ivar output_idfs: A dictionary containing all output IDFs following the format {'output idf filename': class ``eppy.modeleditor.IDF object``}
    :ivar windows_and_doors: A dictionary containing all input idfs and
        window and door names following the format
        {'idf filename': [list of window and door names]}
    :ivar windows_and_doors_original_name: A dictionary containing all input idfs and
        window and door original names following the format
        {'idf filename': [list of window and door original names]}
    """
    def __init__(
        self,
        idfs: list = None,
        ScriptType: str = None,
        SupplyAirTempInputMethod: str = None,
        Output_type: str = None,
        Output_freqs: any = None,
        Output_keep_existing: bool = None,
        Output_gen_dataframe: bool = None,
        Output_take_dataframe: pd.DataFrame = None,
        EnergyPlus_version: str = 'auto',
        TempCtrl: str = None,
        VRFschedule: str = 'On 24/7',
        ComfStand: any = None,
        CAT: any = None,
        CATcoolOffset: float = 0,
        CATheatOffset: float = 0,
        ComfMod: any = None,
        SetpointAcc: float = 10000,
        CustAST_ACSTaul: float = 0,
        CustAST_ACSTall: float = 0,
        CustAST_AHSTaul: float = 0,
        CustAST_AHSTall: float = 0,
        CustAST_m: float = 0,
        CustAST_n: float = 0,
        CustAST_ACSToffset: float = 0,
        CustAST_AHSToffset: float = 0,
        CoolSeasonStart: any = 121,
        CoolSeasonEnd: any = 274,
        HVACmode: any = None,
        VentCtrl: any = None,
        MaxTempDiffVOF: float = 6,
        MinTempDiffVOF: float = 1,
        MultiplierVOF: float = 0.25,
        VSToffset: any = [0],
        MinOToffset: any = [50],
        MaxWindSpeed: any = [50],
        ASTtol_start: float = 0.1,
        ASTtol_end_input: float = 0.1,
        ASTtol_steps: float = 0.1,
        NameSuffix: str = '',
        verboseMode: bool = True,
        confirmGen: bool = None,
        eer: float = 2,
        cop: float = 2.1,
        make_averages: bool = False,
        debugging: bool = False,
    ):
        """
        Constructor method.
        """

        # import accim.sim.accim_Main as accim_Main
        import accim.sim.accim_Main as accim_Main

        from os import listdir, remove
        import accim
        import pandas as pd

        if idfs is None:
            filelist = ([file for file in listdir() if file.endswith('.idf')
                         and not '[' in file
                         and not '_pymod' in file])
        else:
            filelist = [file for file in idfs
                        if file.endswith('.idf')
                        and not '[' in file
                        and not '_pymod' in file]
        if len(filelist) == 0:
            raise FileNotFoundError('No idf files were found. There must be at least 1 idf file located at the path where this script is being run.')

        filelist = ([file.split('.idf')[0] for file in filelist])

        # todo avoid AHST higher than ACST when CAT offsets are used

        objArgsDef = (
            ScriptType is not None,
            SupplyAirTempInputMethod is not None,
            Output_type is not None,
            Output_freqs is not None,
            Output_keep_existing is not None,
            # EnergyPlus_version is not None,
            TempCtrl is not None,
        )

        fullScriptTypeList = [
            'vrf_ac',
            'vrf_mm',
            'ex_mm',
            'ex_ac',
        ]

        SupplyAirTempInputMethodList = [
            'supply air temperature',
            'temperature difference'
        ]

        fullOutputsTypeList = [
            'Standard',
            'standard',
            'Simplified',
            'simplified',
            'Detailed',
            'detailed',
            'Custom',
            'custom',
            # 'Show outputs',
            # 'show outputs'
        ]

        fullOutputsFreqList = [
            'Timestep',
            'timestep',
            'Hourly',
            'hourly',
            'Daily',
            'daily',
            'Monthly',
            'monthly',
            'Runperiod',
            'runperiod'
        ]

        fullTempCtrllist = [
            'temperature',
            'temp',
            'pmv'
        ]

        print(
            '\n--------------------------------------------------------'
            f'\nAdaptive-Comfort-Control-Implemented Model (ACCIM) v{__version__}'
            '\n--------------------------------------------------------'
            '\n\nThis tool allows to apply adaptive setpoint temperatures. '
            '\nFor further information, please read the documentation: '
            '\nhttps://accim.readthedocs.io/en/master/'
            '\nFor a visual understanding of the tool, please visit the following jupyter notebooks:'
            '\n-    Using addAccis() to apply adaptive setpoint temperatures'
            '\nhttps://accim.readthedocs.io/en/master/jupyter_notebooks/addAccis/using_addAccis.html'
            '\n-    Using rename_epw_files() to rename the EPWs for proper data analysis after simulation'
            '\nhttps://accim.readthedocs.io/en/master/jupyter_notebooks/rename_epw_files/using_rename_epw_files.html'
            '\n-    Using runEp() to directly run simulations with EnergyPlus'
            '\nhttps://accim.readthedocs.io/en/master/jupyter_notebooks/runEp/using_runEp.html'
            '\n-    Using the class Table() for data analysis'
            '\nhttps://accim.readthedocs.io/en/master/jupyter_notebooks/Table/using_Table.html'
            '\n-    Full example'
            '\nhttps://accim.readthedocs.io/en/master/jupyter_notebooks/full_example/full_example.html'
            '\n'
            '\nStarting with the process.'
        )
        self.arguments = {}
        if all(objArgsDef):
            pass
        else:
            print(
                '\nNow, you are going to be asked to enter some information for different arguments '
                'to generate the output IDFs with adaptive setpoint temperatures. '
                '\nIf you are not sure about how to use these parameters, please take a look at the documentation in the following link: '
                #todo change url in all places
                '\nhttps://accim.readthedocs.io/en/master/4_detailed%20use.html'
                '\n\nPlease, enter the following information:'
            )
            ScriptType = input("\nEnter the ScriptType (\n"
                               "for VRFsystem with full air-conditioning mode: vrf_ac;\n"
                               "for VRFsystem with mixed-mode: vrf_mm;\n"
                               "for ExistingHVAC with mixed mode: ex_mm;\n"
                               "for ExistingHVAC with full air-conditioning mode: ex_ac\n"
                               "): ")
            while ScriptType not in fullScriptTypeList:
                ScriptType = input("    ScriptType was not correct. "
                                   "    Enter the ScriptType (\n"
                                   "    for VRFsystem with full air-conditioning mode: vrf_ac;\n"
                                   "    for VRFsystem with mixed-mode: vrf_mm;\n"
                                   "    for ExistingHVAC with mixed mode: ex_mm;\n"
                                   "    for ExistingHVAC with full air-conditioning mode: ex_ac\n"
                                   "    ): ")
            if 'vrf' in ScriptType.lower():
                SupplyAirTempInputMethod = input("\nEnter the SupplyAirTempInputMethod (\n"
                                   "for Supply Air Temperature: supply air temperature;\n"
                                   "for Temperature Difference: temperature difference;\n"
                                   "): ")
                while SupplyAirTempInputMethod not in SupplyAirTempInputMethodList:
                    SupplyAirTempInputMethod = input(
                        "    SupplyAirTempInputMethod was not correct. "
                        "    Enter the SupplyAirTempInputMethod (\n"
                                   "for Supply Air Temperature: supply air temperature;\n"
                                   "for Temperature Difference: temperature difference;\n"
                                   "): ")
            Output_keep_existing = input('\nDo you want to keep the existing outputs (true or false)?: ')
            while Output_keep_existing.lower() not in ['true', 'false']:
                Output_keep_existing = input('The answer you entered is not valid. '
                                              'Do you want to keep the existing outputs (true or false)?: ')
            Output_type = input("\nEnter the Output type (standard, simplified, detailed or custom): ")
            while Output_type not in fullOutputsTypeList:
                Output_type = input("   Output type was not correct. "
                                "Please, enter the Output type (standard, simplified, detailed or custom): ")
            Output_freqs = list(freq for freq in input(
                "\nEnter the Output frequencies separated by space (timestep, hourly, daily, monthly, runperiod): ").split())
            while (not(all(elem in fullOutputsFreqList for elem in Output_freqs))):
                Output_freqs = list(freq for freq in input(
                    "Some of the Output frequencies are not correct. "
                    "Please, enter the Output frequencies again separated by space "
                    "(timestep, hourly, daily, monthly, runperiod): ").split())
            Output_gen_dataframe = input('\nDo you want to generate a dataframe to see all outputs? (true or false): ')
            while Output_gen_dataframe.lower() not in ['true', 'false']:
                Output_gen_dataframe = input('The answer you entered is not valid. '
                                              'Do you want to generate a dataframe to see all outputs? (true or false):')
            if Output_gen_dataframe.lower() == 'true':
                Output_gen_dataframe = True
            elif Output_gen_dataframe.lower() == 'false':
                Output_gen_dataframe = False
            EnergyPlus_version = input("\nEnter the EnergyPlus version (9.1 to 25.1, or auto): ")
            while EnergyPlus_version not in lists.fullEPversionsList:
                EnergyPlus_version = input("    EnergyPlus version was not correct. "
                                           "Please, enter the EnergyPlus version (9.1 to 25.1, or auto): ")
            TempCtrl = input('\nEnter the Temperature Control method (temperature or pmv): ')
            while TempCtrl not in fullTempCtrllist:
                TempCtrl = input("  Temperature Control method was not correct. "
                                 "Please, enter the Temperature Control method (temperature or pmv): ")

        if verboseMode:
            print('Basic input data:')
            # print(f'accim version: {accim.__version__}')
            print('ScriptType is: '+ScriptType)
        if ScriptType not in fullScriptTypeList:
            print('Valid ScriptTypes: ')
            print(fullScriptTypeList)
            raise ValueError(ScriptType + " is not a valid ScriptType. "
                                          "You must choose a ScriptType from the list above.")
        if 'vrf' in ScriptType.lower():
            if verboseMode:
                print('Supply Air Temperature Input Method is: '+SupplyAirTempInputMethod)
            if SupplyAirTempInputMethod not in SupplyAirTempInputMethodList:
                print('Valid Supply Air Temperature Input Methods: ')
                print(SupplyAirTempInputMethod)
                raise ValueError(SupplyAirTempInputMethod + " is not a valid Supply Air Temperature Input Method. "
                                              "You must choose a Supply Air Temperature Input Method from the list above.")
        if verboseMode:
            print('Output type is: ' + Output_type)
        if Output_type not in fullOutputsTypeList:
            print('Valid Output type: ')
            print(fullOutputsTypeList)
            raise ValueError(Output_type + " is not a valid Output. "
                                       "You must choose a Output from the list above.")
        if verboseMode:
            print('Output frequencies are: ')
            print(Output_freqs)
        if not (all(elem in fullOutputsFreqList for elem in Output_freqs)):
            print('Valid Output freqs: ')
            print(fullOutputsFreqList)
            raise ValueError('Some of the Output frequencies in '+Output_freqs + " is not a valid Output. "
                                       "All Output frequencies must be included in the list above.")
        if EnergyPlus_version.lower() != 'auto':
            if verboseMode:
                print('EnergyPlus version is: '+EnergyPlus_version)
            if EnergyPlus_version not in lists.fullEPversionsList:
                print('Valid EnergyPlus_version: ')
                print(lists.fullEPversionsList)
                raise ValueError(EnergyPlus_version + " is not a valid EnergyPlus_version. "
                                                      "You must choose a EnergyPlus_version"
                                                      "from the list above.")
        if verboseMode:
            print('Temperature Control method is: '+TempCtrl)
        if TempCtrl not in fullTempCtrllist:
            print('Valid Temperature Control methods: ')
            print(fullTempCtrllist)
            raise ValueError(TempCtrl + " is not a valid Temperature Control method. "
                                                  "You must choose a Temperature Control method"
                                                  "from the list above.")
        self.arguments.update(
            {
                'ScriptType': ScriptType,
                'SupplyAirTempInputMethod': SupplyAirTempInputMethod,
                'Output_type': Output_type,
                'Output_freqs': Output_freqs,
                'Output_keep_existing': Output_keep_existing,
                'Output_gen_dataframe': Output_gen_dataframe,
                'Output_take_dataframe': Output_take_dataframe,
                'EnergyPlus_version': EnergyPlus_version,
                'TempCtrl': TempCtrl,
            }
        )

        notWorkingIDFs = []

        if Output_gen_dataframe:
            df_outputs_to_concat = []
        self.input_idfs = {}
        self.occupied_zones = {}
        self.occupied_zones_original_name = {}
        self.windows_and_doors = {}
        self.windows_and_doors_original_name = {}

        for file in filelist:
            if verboseMode:
                print('''\n=======================START OF GENERIC IDF FILE GENERATION PROCESS=======================\n''')
                print('Starting with file:')
                print(file)
            z = accim_Main.accimJob(
                filename_temp=file,
                ScriptType=ScriptType,
                EnergyPlus_version=EnergyPlus_version,
                TempCtrl=TempCtrl,
                verboseMode=verboseMode
            )

            if EnergyPlus_version.lower() == 'auto':
                EnergyPlus_version = '.'.join([str(i) for i in z.idf1.idd_version[:2]])

            self.input_idfs.update({file: z.idf0})
            self.occupied_zones.update({file: z.occupiedZones})
            self.occupied_zones_original_name.update({file: z.occupiedZones_orig})
            if z.ismixedmode:
                self.windows_and_doors.update({file: z.windownamelist})
                self.windows_and_doors_original_name.update({file: z.windownamelist_orig})

            if z.accimNotWorking is True:
                # raise KeyError(f'accim is not going to work with {file}')
                notWorkingIDFs.append(file)
                continue

            z.setComfFieldsPeople(EnergyPlus_version=EnergyPlus_version, TempCtrl=TempCtrl, verboseMode=verboseMode)

            if 'vrf' in ScriptType.lower():
                if TempCtrl.lower() == 'temperature' or TempCtrl.lower() == 'temp':
                    z.addOpTempTherm(verboseMode=verboseMode)
                elif TempCtrl.lower() == 'pmv':
                    z.setPMVsetpoint(verboseMode=verboseMode)
                z.addBaseSchedules(verboseMode=verboseMode)
                z.setAvailSchOn(verboseMode=verboseMode)
                z.addVRFsystemSch(verboseMode=verboseMode)
                z.addCurveObj(verboseMode=verboseMode)
                z.addDetHVACobj(
                    EnergyPlus_version=EnergyPlus_version,
                    verboseMode=verboseMode,
                    SupplyAirTempInputMethod=SupplyAirTempInputMethod,
                    eer=eer,
                    cop=cop,
                    VRFschedule=VRFschedule
                )
                if ScriptType.lower() == 'vrf_mm':
                    z.checkVentIsOn(verboseMode=verboseMode)
                z.addForscriptSchVRFsystem(verboseMode=verboseMode)
            elif 'ex' in ScriptType.lower():
                # todo check if PMV can work with ex_ac
                z.addForscriptSchExistHVAC(verboseMode=verboseMode)

            z.addEMSProgramsBase(ScriptType=ScriptType, verboseMode=verboseMode)
            z.addEMSOutputVariableBase(ScriptType=ScriptType, verboseMode=verboseMode)
            z.addGlobVarList(ScriptType=ScriptType, verboseMode=verboseMode)
            z.addIntVarList(verboseMode=verboseMode)
            z.addEMSSensorsBase(ScriptType=ScriptType, verboseMode=verboseMode)
            z.addEMSActuatorsBase(ScriptType=ScriptType, verboseMode=verboseMode)

            if 'vrf' in ScriptType.lower():
                z.addEMSSensorsVRFsystem(ScriptType=ScriptType, verboseMode=verboseMode)
            elif ScriptType.lower() == 'ex_mm':
                z.addEMSSensorsExisHVAC(verboseMode=verboseMode)

            z.addEMSPCMBase(verboseMode=verboseMode)

            if make_averages:
                z.makeAverages(verboseMode=verboseMode)

            z.addControlFilesObjects(verboseMode=verboseMode)

            z.addOutputVariableDictionaryObject(verboseMode=verboseMode)

            if debugging:
                z.addOutputEnergyManagementSystem(verboseMode=verboseMode)

            if Output_keep_existing == 'true':
                Output_keep_existing = True
            elif Output_keep_existing == 'false':
                Output_keep_existing = False
            if Output_keep_existing is True:
                pass
            else:
                z.removeExistingOutputVariables()

            if Output_type.lower() == 'simplified':
                z.addOutputVariablesSimplified(
                    Outputs_freq=Output_freqs,
                    TempCtrl=TempCtrl,
                    verboseMode=verboseMode
                )
            elif Output_type.lower() == 'standard':
                z.addOutputVariablesStandard(
                    Outputs_freq=Output_freqs,
                    ScriptType=ScriptType,
                    TempCtrl=TempCtrl,
                    verboseMode=verboseMode
                )
            elif Output_type.lower() == 'detailed' or Output_type.lower() == 'custom':
                z.addOutputVariablesStandard(
                    Outputs_freq=Output_freqs,
                    ScriptType=ScriptType,
                    TempCtrl=TempCtrl,
                    verboseMode=verboseMode
                )
                z.addOutputVariablesDetailed(
                    Outputs_freq=Output_freqs,
                    verboseMode=verboseMode
                )
                if Output_type.lower() == 'custom':
                    Output_gen_dataframe = False
                    z.outputsSpecified()

            if Output_take_dataframe is not None:
                z.takeOutputDataFrame(
                    idf_filename=file,
                    df_outputs_in=Output_take_dataframe,
                    verboseMode=verboseMode
                )

            z.removeDuplicatedOutputVariables()

            if Output_gen_dataframe:
                z.genOutputDataframe(idf_filename=file)
                df_outputs_to_concat.append(z.df_outputs_temp)

            z.saveaccim(verboseMode=verboseMode)
            if verboseMode:
                print('Ending with file:')
                print(file)
                print('''\n=======================END OF GENERIC IDF FILE GENERATION PROCESS=======================\n''')

        if Output_gen_dataframe:
            self.df_outputs = pd.concat(df_outputs_to_concat)

        if verboseMode:
            print('The following IDFs will not work, and therefore these will be deleted:')
        if len(notWorkingIDFs) > 0:
            if verboseMode:
                print(*notWorkingIDFs, sep="\n")
            filelist_pymod = ([file for file in listdir() if file.endswith('.idf')
                         and '_pymod' in file])

            for file in notWorkingIDFs:
                for i in filelist_pymod:
                    if file in i:
                        remove(i)
        else:
            if verboseMode:
                print('None')

        if verboseMode:
            print('''\n=======================START OF OUTPUT IDF FILES GENERATION PROCESS=======================\n''')

        args_needed_mm = (
            ComfStand is not None,
            CAT is not None,
            ComfMod is not None,
            HVACmode is not None,
            VentCtrl is not None,
        )

        args_needed_ac = (
            ComfStand is not None,
            CAT is not None,
            ComfMod is not None,
        )
        if ScriptType.lower() == 'vrf_mm' or ScriptType.lower() == 'ex_mm':
            if all(args_needed_mm):
                z.genIDF(
                    ScriptType=ScriptType,
                    TempCtrl=TempCtrl,
                    ComfStand=ComfStand,
                    CAT=CAT,
                    CATcoolOffset=CATcoolOffset,
                    CATheatOffset=CATheatOffset,
                    ComfMod=ComfMod,
                    SetpointAcc=SetpointAcc,
                    CustAST_ACSTaul=CustAST_ACSTaul,
                    CustAST_ACSTall=CustAST_ACSTall,
                    CustAST_AHSTaul=CustAST_AHSTaul,
                    CustAST_AHSTall=CustAST_AHSTall,
                    CustAST_m=CustAST_m,
                    CustAST_n=CustAST_n,
                    CustAST_ACSToffset=CustAST_ACSToffset,
                    CustAST_AHSToffset=CustAST_AHSToffset,
                    CoolSeasonStart=CoolSeasonStart,
                    CoolSeasonEnd=CoolSeasonEnd,
                    HVACmode=HVACmode,
                    VentCtrl=VentCtrl,
                    MaxTempDiffVOF=MaxTempDiffVOF,
                    MinTempDiffVOF=MinTempDiffVOF,
                    MultiplierVOF=MultiplierVOF,
                    VSToffset=VSToffset,
                    MinOToffset=MinOToffset,
                    MaxWindSpeed=MaxWindSpeed,
                    ASTtol_start=ASTtol_start,
                    ASTtol_end_input=ASTtol_end_input,
                    ASTtol_steps=ASTtol_steps,
                    NameSuffix=NameSuffix,
                    verboseMode=verboseMode,
                    confirmGen=confirmGen
                    )
                self.arguments.update(
                    {
                        'ScriptType': ScriptType,
                        'TempCtrl': TempCtrl,
                        'ComfStand': ComfStand,
                        'CAT': CAT,
                        'CATcoolOffset': CATcoolOffset,
                        'CATheatOffset': CATheatOffset,
                        'ComfMod': ComfMod,
                        'SetpointAcc': SetpointAcc,
                        'CustAST_ACSTaul': CustAST_ACSTaul,
                        'CustAST_ACSTall': CustAST_ACSTall,
                        'CustAST_AHSTaul': CustAST_AHSTaul,
                        'CustAST_AHSTall': CustAST_AHSTall,
                        'CustAST_m': CustAST_m,
                        'CustAST_n': CustAST_n,
                        'CustAST_ACSToffset': CustAST_ACSToffset,
                        'CustAST_AHSToffset': CustAST_AHSToffset,
                        'CoolSeasonStart': CoolSeasonStart,
                        'CoolSeasonEnd': CoolSeasonEnd,
                        'HVACmode': HVACmode,
                        'VentCtrl': VentCtrl,
                        'MaxTempDiffVOF': MaxTempDiffVOF,
                        'MinTempDiffVOF': MinTempDiffVOF,
                        'MultiplierVOF': MultiplierVOF,
                        'VSToffset': VSToffset,
                        'MinOToffset': MinOToffset,
                        'MaxWindSpeed': MaxWindSpeed,
                        'ASTtol_start': ASTtol_start,
                        'ASTtol_end_input': ASTtol_end_input,
                        'ASTtol_steps': ASTtol_steps,
                        'NameSuffix': NameSuffix,
                        'verboseMode': verboseMode,
                        'confirmGen': confirmGen,
                    }
                )
            else:
                z.inputData(
                    ScriptType=ScriptType,
                )
                self.arguments.update(z.user_input_arguments)
                self.arguments.update(
                    {
                        'NameSuffix': NameSuffix,
                        'verboseMode': verboseMode,
                        'confirmGen': confirmGen,
                    }
                )
                z.genIDF(
                    ScriptType=ScriptType,
                    TempCtrl=TempCtrl,
                )
        elif ScriptType.lower() == 'ex_ac' or ScriptType.lower() == 'vrf_ac':
            if all(args_needed_ac):
                z.genIDF(
                    ScriptType=ScriptType,
                    TempCtrl=TempCtrl,
                    ComfStand=ComfStand,
                    CAT=CAT,
                    CATcoolOffset=CATcoolOffset,
                    CATheatOffset=CATheatOffset,
                    ComfMod=ComfMod,
                    SetpointAcc=SetpointAcc,
                    CustAST_ACSTaul=CustAST_ACSTaul,
                    CustAST_ACSTall=CustAST_ACSTall,
                    CustAST_AHSTaul=CustAST_AHSTaul,
                    CustAST_AHSTall=CustAST_AHSTall,
                    CustAST_m=CustAST_m,
                    CustAST_n=CustAST_n,
                    CustAST_ACSToffset=CustAST_ACSToffset,
                    CustAST_AHSToffset=CustAST_AHSToffset,
                    CoolSeasonStart=CoolSeasonStart,
                    CoolSeasonEnd=CoolSeasonEnd,
                    HVACmode=[0],
                    VentCtrl=[0],
                    VSToffset=[0],
                    MaxTempDiffVOF=1,
                    MinTempDiffVOF=0,
                    MultiplierVOF=0,
                    MinOToffset=[0],
                    MaxWindSpeed=[0],
                    ASTtol_start=ASTtol_start,
                    ASTtol_end_input=ASTtol_end_input,
                    ASTtol_steps=ASTtol_steps,
                    NameSuffix=NameSuffix,
                    verboseMode=verboseMode,
                    confirmGen=confirmGen
                    )
                self.arguments.update(
                    {
                        'ScriptType': ScriptType,
                        'TempCtrl': TempCtrl,
                        'ComfStand': ComfStand,
                        'CAT': CAT,
                        'CATcoolOffset': CATcoolOffset,
                        'CATheatOffset': CATheatOffset,
                        'ComfMod': ComfMod,
                        'SetpointAcc': SetpointAcc,
                        'CustAST_ACSTaul': CustAST_ACSTaul,
                        'CustAST_ACSTall': CustAST_ACSTall,
                        'CustAST_AHSTaul': CustAST_AHSTaul,
                        'CustAST_AHSTall': CustAST_AHSTall,
                        'CustAST_m': CustAST_m,
                        'CustAST_n': CustAST_n,
                        'CustAST_ACSToffset': CustAST_ACSToffset,
                        'CustAST_AHSToffset': CustAST_AHSToffset,
                        'CoolSeasonStart': CoolSeasonStart,
                        'CoolSeasonEnd': CoolSeasonEnd,
                        'HVACmode': [0],
                        'VentCtrl': [0],
                        'MaxTempDiffVOF': 1,
                        'MinTempDiffVOF': 0,
                        'MultiplierVOF': 0,
                        'VSToffset': [0],
                        'MinOToffset': [0],
                        'MaxWindSpeed': [0],
                        'ASTtol_start': ASTtol_start,
                        'ASTtol_end_input': ASTtol_end_input,
                        'ASTtol_steps': ASTtol_steps,
                        'NameSuffix': NameSuffix,
                        'verboseMode': verboseMode,
                        'confirmGen': confirmGen,
                    }
                )
            else:
                z.inputData(
                    ScriptType=ScriptType,
                )
                self.arguments.update(z.user_input_arguments)
                self.arguments.update(
                    {
                        'NameSuffix': NameSuffix,
                        'verboseMode': verboseMode,
                        'confirmGen': confirmGen,
                    }
                )
                z.genIDF(
                    ScriptType=ScriptType,
                    TempCtrl=TempCtrl,
                )
        self.output_idfs = z.output_idf_dict
        if verboseMode:
            print('''\n=======================END OF OUTPUT IDF FILES GENERATION PROCESS=======================\n''')

        #todo pop up when process ends; by defalt True



# accim - Adaptive-Comfort-Control-Implemented Model
# Copyright (C) 2021-2025 Daniel Sánchez-García

# accim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# accim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import besos.IDF_class

# def modify_VSToffset(building, value):
#     import accim.sim.accis_single_idf_funcs as accis
#     accis.modifyAccis(
#         idf=building,
#         ComfStand=1,
#         CAT=3,
#         ComfMod=3,
#         # SetpointAcc=1000,
#         HVACmode=2,
#         VentCtrl=0,
#         CoolSeasonStart='01/02',
#         CoolSeasonEnd='01/03',
#         VSToffset=value,
#         # MinOToffset=50,
#         # MaxWindSpeed=50
#     )
#     return


def get_valid_param_combinations():

    # CS_CA_CM_list_dict = {
    #     0:{
    #         'name': '0 = ESP CTE',
    #         'CAT': ['n/a'],
    #         'ComfMod': ['n/a'],
    #     },
    #     0: {
    #         'name': ,
    #         'CAT': ,
    #         'ComfMod': ,
    #     },
    #
    # }
    #     '0 = ESP CTE': [, ],
    #     '1 = INT EN16798-1': [[1, 2, 3], [0, 1, 2, 3]],
    #     '2 = INT ASHRAE55': [[80, 90], [0, 1, 2, 3]],
    #     '3 = JPN Rijal': [[80, 90], [0, 1, 2, 3]],
    #     '4 = CHN GBT50785 Cold': [[1, 2], [3]],
    #     '5 = CHN GBT50785 HotMild': [[1, 2], [3]],
    #     '6 = CHN Yang': [[80, 90], [0, 1, 2, 3]],
    #     '7 = IND IMAC C NV': [[80, 85, 90], [0, 1, 2, 3]],
    #     '8 = IND IMAC C MM': [[80, 85, 90], [0, 1, 2, 3]],
    #     '9 = IND IMAC R 7DRM': [[80, 90], [0, 1, 2, 3]],
    #     '10 = IND IMAC R 30DRM': [[80, 90], [0, 1, 2, 3]],
    #     '11 = IND Dhaka': [[80, 90], [0, 1, 2, 3]],
    #     '12 = ROM Udrea': [[80, 90], [0, 1, 2, 3]],
    #     '13 = AUS Williamson': [[80, 90], [0.1, 0.2, 0.3, 0.4, 0.5, 1.1, 1.2, 1.3, 1.4, 1.5, 2, 3]],
    #     '14 = AUS DeDear': [[80, 90], [0.1, 0.2, 0.3, 0.4, 0.5, 1.1, 1.2, 1.3, 1.4, 1.5, 2, 3]],
    #     '15 = BRA Rupp NV': [[80, 90], [0, 1, 2, 3]],
    #     '16 = BRA Rupp AC': [[80, 90], [0, 1, 2, 3]],
    #     '17 = MEX Oropeza Arid': [[80, 90], [0, 1, 2, 3]],
    #     '18 = MEX Oropeza DryTropic': [[80, 90], [0, 1, 2, 3]],
    #     '19 = MEX Oropeza Temperate': [[80, 90], [0, 1, 2, 3]],
    #     '20 = MEX Oropeza HumTropic': [[80, 90], [0, 1, 2, 3]],
    #     '21 = CHL Perez-Fargallo': [[80, 90], [2, 3]],
    #     '22 = INT ISO7730': [[1, 2, 3], [0]],
    # }

    CS_CA_CM_list_dict = {
        0: [['n/a'], ['n/a']],
        1: [[1, 2, 3], [0, 1, 2, 3]],
        2: [[80, 90], [0, 1, 2, 3]],
        3: [[80, 90], [0, 1, 2, 3]],
        4: [[1, 2], [3]],
        5: [[1, 2], [3]],
        6: [[80, 90], [0, 1, 2, 3]],
        7: [[80, 85, 90], [0, 1, 2, 3]],
        8: [[80, 85, 90], [0, 1, 2, 3]],
        9: [[80, 90], [0, 1, 2, 3]],
        10: [[80, 90], [0, 1, 2, 3]],
        11: [[80, 90], [0, 1, 2, 3]],
        12: [[80, 90], [0, 1, 2, 3]],
        13: [[80, 90], [0.1, 0.2, 0.3, 0.4, 0.5, 1.1, 1.2, 1.3, 1.4, 1.5, 2, 3]],
        14: [[80, 90], [0.1, 0.2, 0.3, 0.4, 0.5, 1.1, 1.2, 1.3, 1.4, 1.5, 2, 3]],
        15: [[80, 90], [0, 1, 2, 3]],
        16: [[80, 90], [0, 1, 2, 3]],
        17: [[80, 90], [0, 1, 2, 3]],
        18: [[80, 90], [0, 1, 2, 3]],
        19: [[80, 90], [0, 1, 2, 3]],
        20: [[80, 90], [0, 1, 2, 3]],
        21: [[80, 90], [2, 3]],
        22: [[1, 2, 3], [0]],
    }

    return CS_CA_CM_list_dict

def drop_invalid_param_combinations(samples_df):
    samples_df = samples_df.dropna()

    valid_params = {
        0: [['n/a'], ['n/a']],
        1: [[1, 2, 3], [0, 1, 2, 3]],
        2: [[80, 90], [0, 1, 2, 3]],
        3: [[80, 90], [0, 1, 2, 3]],
        4: [[1, 2], [3]],
        5: [[1, 2], [3]],
        6: [[80, 90], [0, 1, 2, 3]],
        7: [[80, 85, 90], [0, 1, 2, 3]],
        8: [[80, 85, 90], [0, 1, 2, 3]],
        9: [[80, 90], [0, 1, 2, 3]],
        10: [[80, 90], [0, 1, 2, 3]],
        11: [[80, 90], [0, 1, 2, 3]],
        12: [[80, 90], [0, 1, 2, 3]],
        13: [[80, 90], [0.1, 0.2, 0.3, 0.4, 0.5, 1.1, 1.2, 1.3, 1.4, 1.5, 2, 3]],
        14: [[80, 90], [0.1, 0.2, 0.3, 0.4, 0.5, 1.1, 1.2, 1.3, 1.4, 1.5, 2, 3]],
        15: [[80, 90], [0, 1, 2, 3]],
        16: [[80, 90], [0, 1, 2, 3]],
        17: [[80, 90], [0, 1, 2, 3]],
        18: [[80, 90], [0, 1, 2, 3]],
        19: [[80, 90], [0, 1, 2, 3]],
        20: [[80, 90], [0, 1, 2, 3]],
        21: [[80, 90], [2, 3]],
        22: [[1, 2, 3], [0]],
    }

    samples_df['valid'] = True
    for i in samples_df.index:
        try:
            if samples_df.loc[i, 'CAT'] not in valid_params[samples_df.loc[i, 'ComfStand']][0]:
                samples_df.loc[i, 'valid'] = False
        except KeyError:
            continue

        try:
            if samples_df.loc[i, 'ComfMod'] not in valid_params[samples_df.loc[i, 'ComfStand']][1]:
                samples_df.loc[i, 'valid'] = False
        except KeyError:
            continue

        try:
            if samples_df.loc[i, 'VentCtrl'] != 0 and samples_df.loc[i, 'HVACmode'] == 0:
                samples_df.loc[i, 'valid'] = False
        except KeyError:
            continue

        try:
            if samples_df.loc[i, 'VSToffset'] != 0 and samples_df.loc[i, 'HVACmode'] == 0:
                samples_df.loc[i, 'valid'] = False
        except KeyError:
            continue

        try:
            if samples_df.loc[i, 'MinOToffset'] != 0 and samples_df.loc[i, 'HVACmode'] == 0:
                samples_df.loc[i, 'valid'] = False
        except KeyError:
            continue

        try:
            if samples_df.loc[i, 'MaxWindSpeed'] != 0 and samples_df.loc[i, 'HVACmode'] == 0:
                samples_df.loc[i, 'valid'] = False
        except KeyError:
            continue

        try:
            if samples_df.loc[i, 'SetpointAcc'] < 0:
                samples_df.loc[i, 'valid'] = False
        except KeyError:
            continue
        try:
            if samples_df.loc[i, 'MaxTempDiffVOF'] <= 0:
                samples_df.loc[i, 'valid'] = False
        except KeyError:
            continue

        try:
            if samples_df.loc[i, 'MinTempDiffVOF'] <= 0:
                samples_df.loc[i, 'valid'] = False
        except KeyError:
            continue
        try:
            if samples_df.loc[i, 'MinTempDiffVOF'] >= samples_df.loc[i, 'MaxTempDiffVOF'] <= 0:
                samples_df.loc[i, 'valid'] = False
        except KeyError:
            continue

        try:
            if samples_df.loc[i, 'MultiplierVOF'] < 0 or samples_df.loc[i, 'MultiplierVOF'] > 1:
                samples_df.loc[i, 'valid'] = False
        except KeyError:
            continue


    samples_cleaned = samples_df[samples_df['valid'] == True].drop(columns=['valid'])

    return samples_cleaned


def modify_ComfStand(idf: besos.IDF_class, value: int):
    """
    Modifies the ComfStand argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetInputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                     program.Name == 'SetInputData'][0])
    SetInputData.Program_Line_1 = f'set ComfStand = {value}'

    return

def modify_CustAST_ACSTaul(idf: besos.IDF_class, value: float):
    """
    Modifies the CustAST_ACSTaul argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetAppLimits = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                          program.Name == 'SetAppLimits'][0])
    SetAppLimits.Program_Line_2 = f'set ACSTaul = {value}'

    return

def modify_CustAST_ACSTall(idf: besos.IDF_class, value: float):
    """
    Modifies the CustAST_ACSTall argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetAppLimits = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                          program.Name == 'SetAppLimits'][0])
    SetAppLimits.Program_Line_3 = f'set ACSTall = {value}'

    return

def modify_CustAST_AHSTaul(idf: besos.IDF_class, value: float):
    """
    Modifies the CustAST_AHSTaul argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetAppLimits = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                          program.Name == 'SetAppLimits'][0])
    SetAppLimits.Program_Line_4 = f'set AHSTaul = {value}'

    return

def modify_CustAST_AHSTall(idf: besos.IDF_class, value: float):
    """
    Modifies the CustAST_AHSTall argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetAppLimits = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                          program.Name == 'SetAppLimits'][0])
    SetAppLimits.Program_Line_5 = f'set AHSTall = {value}'

    return

def modify_CustAST_ASTall(idf: besos.IDF_class, value: float):
    """
    Modifies the CustAST_ACSTall and CustAST_AHSTall arguments to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetAppLimits = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                          program.Name == 'SetAppLimits'][0])
    SetAppLimits.Program_Line_3 = f'set ACSTall = {value}'
    SetAppLimits.Program_Line_5 = f'set AHSTall = {value}'

    return

def modify_CustAST_ASTaul(idf: besos.IDF_class, value: float):
    """
    Modifies the CustAST_ACSTaul and CustAST_AHSTaul arguments to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetAppLimits = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                          program.Name == 'SetAppLimits'][0])
    SetAppLimits.Program_Line_2 = f'set ACSTaul = {value}'
    SetAppLimits.Program_Line_4 = f'set AHSTaul = {value}'

    return

def modify_CustAST_m(idf: besos.IDF_class, value: float):
    """
    Modifies the CustAST_m argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetAST = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
               program.Name == 'SetAST'][0])
    SetAST.Program_Line_2 = f'set m = {value}'

    return


def modify_CustAST_n(idf: besos.IDF_class, value: float):
    """
    Modifies the CustAST_n argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetAST = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
               program.Name == 'SetAST'][0])
    SetAST.Program_Line_3 = f'set n = {value}'

    return

def modify_CustAST_ACSToffset(idf: besos.IDF_class, value: float):
    """
    Modifies the ACSToffset argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    ApplyCAT = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                 program.Name == 'ApplyCAT'][0])
    ApplyCAT.Program_Line_4 = f'set ACSToffset = {value}'

    return


def modify_CustAST_AHSToffset(idf: besos.IDF_class, value: float):
    """
    Modifies the AHSToffset argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    ApplyCAT = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                 program.Name == 'ApplyCAT'][0])
    ApplyCAT.Program_Line_5 = f'set AHSToffset = {value}'

    return

def modify_CustAST_ASToffset(idf: besos.IDF_class, value: float):
    """
    Modifies the ACSToffset and AHSToffset arguments symmetrically to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    ApplyCAT = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                 program.Name == 'ApplyCAT'][0])
    ApplyCAT.Program_Line_4 = f'set ACSToffset = {value}'
    ApplyCAT.Program_Line_5 = f'set AHSToffset = {-value}'

    return


def modify_CAT(idf: besos.IDF_class, value: int):
    """
    Modifies the CAT argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetInputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                     program.Name == 'SetInputData'][0])
    SetInputData.Program_Line_2 = f'set CAT = {value}'

    return

def modify_CATcoolOffset(idf: besos.IDF_class, value: float):
    """
    Modifies the CATcoolOffset argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    ApplyCAT = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                 program.Name == 'ApplyCAT'][0])
    ApplyCAT.Program_Line_1 = f'set CATcoolOffset = {value}'

    return

def modify_CATheatOffset(idf: besos.IDF_class, value: float):
    """
    Modifies the CATheatOffset argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    ApplyCAT = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                 program.Name == 'ApplyCAT'][0])
    ApplyCAT.Program_Line_2 = f'set CATheatOffset = {value}'

    return

def modify_ComfMod(idf: besos.IDF_class, value: float):
    """
    Modifies the ComfMod argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetInputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                     program.Name == 'SetInputData'][0])
    SetInputData.Program_Line_3 = f'set ComfMod = {value}'

    return

def modify_HVACmode(idf: besos.IDF_class, value: int):
    """
    Modifies the HVACmode argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetInputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                     program.Name == 'SetInputData'][0])
    SetInputData.Program_Line_4 = f'set HVACmode = {value}'
    return

def modify_VentCtrl(idf: besos.IDF_class, value: int):
    """
    Modifies the VentCtrl argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetInputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                     program.Name == 'SetInputData'][0])
    SetInputData.Program_Line_5 = f'set VentCtrl = {value}'
    return

def modify_VSToffset(idf: besos.IDF_class, value: float):
    """
    Modifies the VSToffset argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetInputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                     program.Name == 'SetInputData'][0])
    SetInputData.Program_Line_6 = f'set VSToffset = {value}'
    return

def modify_MinOToffset(idf: besos.IDF_class, value: float):
    """
    Modifies the MinOToffset argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetInputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                     program.Name == 'SetInputData'][0])
    SetInputData.Program_Line_7 = f'set MinOToffset = {value}'
    return

def modify_MaxWindSpeed(idf: besos.IDF_class, value: float):
    """
    Modifies the MaxWindSpeed argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetInputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                     program.Name == 'SetInputData'][0])
    SetInputData.Program_Line_8 = f'set MaxWindSpeed = {value}'
    return

def modify_ASTtol(idf: besos.IDF_class, value: float):
    """
    Modifies the ACSTtol and AHSTtol arguments symmetrically to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetInputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                     program.Name == 'SetInputData'][0])
    SetInputData.Program_Line_9 = f'set ACSTtol = {-value}'
    SetInputData.Program_Line_10 = f'set AHSTtol = {value}'
    return

def modify_CoolSeasonStart(idf: besos.IDF_class, value: any):
    """
    Modifies the CoolSeasonStart argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument. Can be an integer representing the day of the year, or the
        date in format dd/mm.
    :return:
    """
    SetInputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                     program.Name == 'SetInputData'][0])
    if type(value) is str:
        from accim.utils import transform_ddmm_to_int
        value = transform_ddmm_to_int(value)

    SetInputData.Program_Line_11 = f'set CoolSeasonStart = {value}'
    return

def modify_CoolSeasonEnd(idf: besos.IDF_class, value: any):
    """
    Modifies the CoolSeasonEnd argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument. Can be an integer representing the day of the year, or the
        date in format dd/mm.
    :return:
    """
    SetInputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                     program.Name == 'SetInputData'][0])
    if type(value) is str:
        from accim.utils import transform_ddmm_to_int
        value = transform_ddmm_to_int(value)

    SetInputData.Program_Line_12 = f'set CoolSeasonEnd = {value}'
    return

def modify_SetpointAcc(idf: besos.IDF_class, value: float):
    """
    Modifies the SetpointAcc argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetAST = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
               program.Name == 'SetAST'][0])
    SetAST.Program_Line_1 = f'set SetpointAcc = {value}'
    return

def modify_MaxTempDiffVOF(idf: besos.IDF_class, value: float):
    """
    Modifies the MaxTempDiffVOF argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetVOFinputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                        program.Name == 'SetVOFinputData'][0])
    SetVOFinputData.Program_Line_1 = f'set MaxTempDiffVOF = {value}'
    return

def modify_MinTempDiffVOF(idf: besos.IDF_class, value: float):
    """
    Modifies the MinTempDiffVOF argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetVOFinputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                        program.Name == 'SetVOFinputData'][0])
    SetVOFinputData.Program_Line_2 = f'set MinTempDiffVOF = {value}'
    return

def modify_MultiplierVOF(idf: besos.IDF_class, value: int):
    """
    Modifies the MultiplierVOF argument to match the entered value.

    :param idf: The eppy or besos IDF class instance.
    :param value: The value to be applied in the argument.
    :return:
    """
    SetVOFinputData = ([program for program in idf.idfobjects['EnergyManagementSystem:Program'] if
                        program.Name == 'SetVOFinputData'][0])
    SetVOFinputData.Program_Line_3 = f'set MultiplierVOF = {value}'
    return





from besos import eppy_funcs as ef
from besos.evaluator import EvaluatorEP
from besos.optimizer import NSGAII
from besos.parameters import RangeParameter,  Parameter, GenericSelector
from besos.problem import EPProblem
from besos.objectives import MeterReader

import accim.sim.accis_single_idf_funcs as accis
import accim.parametric_and_optimisation.funcs_for_besos.param_accis as bf

##

building = ef.get_building('TestModel_onlyGeometryForVRFsystem_2zones_CalcVent_V2310.idf')

accis.addAccis(
    idf=building,
    ScriptType='vrf_mm',
    SupplyAirTempInputMethod='temperature difference',
    Output_keep_existing=False,
    Output_type='standard',
    Output_freqs=['hourly'],
    # EnergyPlus_version='9.4',
    TempCtrl='temperature',
    Output_gen_dataframe=True,
)

##

# def modify_accis(building, value):
#     accis.modifyAccis(
#         idf=building,
#         ComfStand=1,
#         CAT=3,
#         ComfMod=3,
#         # SetpointAcc=1000,
#         HVACmode=2,
#         VentCtrl=0,
#         CoolSeasonStart='01/02',
#         CoolSeasonEnd='01/03',
#         VSToffset=value,
#         # MinOToffset=50,
#         # MaxWindSpeed=50
#     )
#     return

##
parameters_set = [
    Parameter(
        name='VSToffset',
        selector=GenericSelector(set=bf.modify_VSToffset),
        value_descriptor=RangeParameter(min_val=0.1, max_val=0.9)
    ),
]

##

objectives = [
    MeterReader("Electricity:Facility", name="Electricity Usage"),
]

evaluator = EvaluatorEP(
    problem=EPProblem(
        inputs=parameters_set,
        outputs=objectives,
        minimize_outputs=[
            True
        ],
    ),
    building=building,
)


results1 = NSGAII(evaluator, evaluations=1, population_size=3)



# accim - Adaptive-Comfort-Control-Implemented Model
# Copyright (C) 2021-2025 Daniel Sánchez-García

# accim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# accim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

from accim.parametric_and_optimisation.funcs_for_besos import param_accis, param_apmv
import accim.parametric_and_optimisation.params_dicts as params_dicts
from accim.parametric_and_optimisation.utils import descriptor_has_options

def accis_parameter(parameter_name, values):
    desc_has_options = descriptor_has_options(values)

    # all_params = {
    #     # accim predefined models parameters
    #     'ComfStand': param_accis.modify_ComfStand,
    #     'CAT': param_accis.modify_CAT,
    #     'CATcoolOffset': param_accis.modify_CATcoolOffset,
    #     'CATheatOffset': param_accis.modify_CATheatOffset,
    #     'ComfMod': param_accis.modify_ComfMod,
    #     'SetpointAcc': param_accis.modify_SetpointAcc,
    #     'CoolSeasonStart': param_accis.modify_CoolSeasonStart,
    #     'CoolSeasonEnd': param_accis.modify_CoolSeasonEnd,
    #     'HVACmode': param_accis.modify_HVACmode,
    #     'VentCtrl': param_accis.modify_VentCtrl,
    #     'MaxTempDiffVOF': param_accis.modify_MaxTempDiffVOF,
    #     'MinTempDiffVOF': param_accis.modify_MinTempDiffVOF,
    #     'MultiplierVOF': param_accis.modify_MultiplierVOF,
    #     'VSToffset': param_accis.modify_VSToffset,
    #     'MinOToffset': param_accis.modify_MinOToffset,
    #     'MaxWindSpeed': param_accis.modify_MaxWindSpeed,
    #     'ASTtol': param_accis.modify_ASTtol,
    #     # accim custom models parameters
    #     'CustAST_ACSTaul': param_accis.modify_CustAST_ACSTaul,
    #     'CustAST_ACSTall': param_accis.modify_CustAST_ACSTall,
    #     'CustAST_AHSTaul': param_accis.modify_CustAST_AHSTaul,
    #     'CustAST_AHSTall': param_accis.modify_CustAST_AHSTall,
    #     'CustAST_ASTaul': param_accis.modify_CustAST_ASTaul,
    #     'CustAST_ASTall': param_accis.modify_CustAST_ASTall,
    #     'CustAST_m': param_accis.modify_CustAST_m,
    #     'CustAST_n': param_accis.modify_CustAST_n,
    #     'CustAST_ACSToffset': param_accis.modify_CustAST_ACSToffset,
    #     'CustAST_AHSToffset': param_accis.modify_CustAST_AHSToffset,
    #     'CustAST_ASToffset': param_accis.modify_CustAST_ASToffset,
    #     #apmv setpoints parameters
    #     'Adaptive coefficient': param_apmv.change_adaptive_coeff_all_zones,
    #     'Adaptive cooling coefficient': param_apmv.change_adaptive_coeff_cooling_all_zones,
    #     'Adaptive heating coefficient': param_apmv.change_adaptive_coeff_heating_all_zones,
    #     'PMV setpoint': param_apmv.change_pmv_setpoint_all_zones,
    #     'PMV cooling setpoint': param_apmv.change_pmv_cooling_setpoint_all_zones,
    #     'PMV heating setpoint': param_apmv.change_pmv_heating_setpoint_all_zones,
    # }


    if parameter_name.lower() not in [k.lower() for k in params_dicts.all_params.keys()]:
        raise KeyError(f'Parameter do not exist.'
                       f'You need to chose one of the following list: {params_dicts.all_params.keys()}')

    name = [i for i in params_dicts.all_params.keys() if i.lower() == parameter_name.lower()][0]

    from besos.parameters import Parameter, GenericSelector, CategoryParameter, RangeParameter
    import accim.parametric_and_optimisation.funcs_for_besos.param_accis as bf
    import numpy as np

    if desc_has_options:
        parameter = Parameter(
            name=name,
            # selector=GenericSelector(set=change_adaptive_coeff),
            selector=GenericSelector(set=params_dicts.all_params[name]),
            # value_descriptors=RangeParameter(name='CustAST_m', min_val=0, max_val=0.7),
            value_descriptors=CategoryParameter(
                name=name,
                options=values
            ),
        ),
    else:
        parameter = Parameter(
            name=name,
            # selector=GenericSelector(set=change_adaptive_coeff),
            selector=GenericSelector(set=params_dicts.all_params[name]),
            # value_descriptors=RangeParameter(name='CustAST_m', min_val=0, max_val=0.7),
            value_descriptors=RangeParameter(
                name=name,
                min_val=values[0],
                max_val=values[1],
            ),
        ),

    return parameter[0]

def get_available_params_accim_predef_models():
    param_dict = [k for k in params_dicts.accim_predef_model_params.keys()]
    return param_dict
def get_available_params_accim_custom_models():
    param_dict = [k for k in params_dicts.accim_custom_model_params.keys()]
    return param_dict
def get_available_params_apmv_setpoints():
    param_dict = [k for k in params_dicts.apmv_setpoints_params.keys()]
    return param_dict

class Parameter:
    def __init__(self, parameter):
        parameters_accis = {
            'ComfStand': param_accis.modify_ComfStand,
            'CAT': param_accis.modify_CAT,
            'CATcoolOffset': param_accis.modify_CATcoolOffset,
            'CATheatOffset': param_accis.modify_CATheatOffset,
            'ComfMod': param_accis.modify_ComfMod,
            'SetpointAcc': param_accis.modify_SetpointAcc,
            'CustAST_ACSTaul': param_accis.modify_CustAST_ACSTaul,
            'CustAST_ACSTall': param_accis.modify_CustAST_ACSTall,
            'CustAST_AHSTaul': param_accis.modify_CustAST_AHSTaul,
            'CustAST_AHSTall': param_accis.modify_CustAST_AHSTall,
            'CustAST_m': param_accis.modify_CustAST_m,
            'CustAST_n': param_accis.modify_CustAST_n,
            'CustAST_ACSToffset': param_accis.modify_CustAST_ACSToffset,
            'CustAST_AHSToffset': param_accis.modify_CustAST_AHSToffset,
            'CoolSeasonStart': param_accis.modify_CoolSeasonStart,
            'CoolSeasonEnd': param_accis.modify_CoolSeasonEnd,
            'HVACmode': param_accis.modify_HVACmode,
            'VentCtrl': param_accis.modify_VentCtrl,
            'MaxTempDiffVOF': param_accis.modify_MaxTempDiffVOF,
            'MinTempDiffVOF': param_accis.modify_MinTempDiffVOF,
            'MultiplierVOF': param_accis.modify_MultiplierVOF,
            'VSToffset': param_accis.modify_VSToffset,
            'MinOToffset': param_accis.modify_MinOToffset,
            'MaxWindSpeed': param_accis.modify_MaxWindSpeed,
            'ASTtol': param_accis.modify_ASTtol,
        }

        if parameter.lower() not in [k.lower() for k in parameters_accis.keys()]:
            raise KeyError(f'Parameter do not exist.'
                           f'You need to chose one of the following list: {parameters_accis.keys()}')

        self.name = [i for i in parameters_accis.keys() if i.lower() == parameter.lower()][0]

    def modify(self, idf, value):
        parameters_accis = {
            'ComfStand': param_accis.modify_ComfStand(idf, value),
            'CAT': param_accis.modify_CAT(idf, value),
            'CATcoolOffset': param_accis.modify_CATcoolOffset(idf, value),
            'CATheatOffset': param_accis.modify_CATheatOffset(idf, value),
            'ComfMod': param_accis.modify_ComfMod(idf, value),
            'SetpointAcc': param_accis.modify_SetpointAcc(idf, value),
            'CustAST_ACSTaul': param_accis.modify_CustAST_ACSTaul(idf, value),
            'CustAST_ACSTall': param_accis.modify_CustAST_ACSTall(idf, value),
            'CustAST_AHSTaul': param_accis.modify_CustAST_AHSTaul(idf, value),
            'CustAST_AHSTall': param_accis.modify_CustAST_AHSTall(idf, value),
            'CustAST_m': param_accis.modify_CustAST_m(idf, value),
            'CustAST_n': param_accis.modify_CustAST_n(idf, value),
            'CustAST_ACSToffset': param_accis.modify_CustAST_ACSToffset(idf, value),
            'CustAST_AHSToffset': param_accis.modify_CustAST_AHSToffset(idf, value),
            'CoolSeasonStart': param_accis.modify_CoolSeasonStart(idf, value),
            'CoolSeasonEnd': param_accis.modify_CoolSeasonEnd(idf, value),
            'HVACmode': param_accis.modify_HVACmode(idf, value),
            'VentCtrl': param_accis.modify_VentCtrl(idf, value),
            'MaxTempDiffVOF': param_accis.modify_MaxTempDiffVOF(idf, value),
            'MinTempDiffVOF': param_accis.modify_MinTempDiffVOF(idf, value),
            'MultiplierVOF': param_accis.modify_MultiplierVOF(idf, value),
            'VSToffset': param_accis.modify_VSToffset(idf, value),
            'MinOToffset': param_accis.modify_MinOToffset(idf, value),
            'MaxWindSpeed': param_accis.modify_MaxWindSpeed(idf, value),
            'ASTtol': param_accis.modify_ASTtol(idf, value),
        }

        parameters_accis[self.name]


class ComfStand:
    def __init__(self):
        self.name = 'ComfStand'

    def modify(self, idf, value):
        param_accis.modify_ComfStand(idf, value)

class CustAST_ACSTaul:
    def __init__(self):
        self.name = 'CustAST_ACSTaul'

    def modify(self, idf, value):
        param_accis.modify_CustAST_ACSTaul(idf, value)

class CustAST_ACSTall:
    def __init__(self):
        self.name = 'CustAST_ACSTall'

    def modify(self, idf, value):
        param_accis.modify_CustAST_ACSTall(idf, value)

class CustAST_AHSTaul:
    def __init__(self):
        self.name = 'CustAST_AHSTaul'

    def modify(self, idf, value):
        param_accis.modify_CustAST_AHSTaul(idf, value)

class CustAST_AHSTall:
    def __init__(self):
        self.name = 'CustAST_AHSTall'

    def modify(self, idf, value):
        param_accis.modify_CustAST_AHSTall(idf, value)

class CustAST_ASTaul:
    def __init__(self):
        self.name = 'CustAST_ASTaul'

    def modify(self, idf, value):
        param_accis.modify_CustAST_ASTaul(idf, value)

class CustAST_ASTall:
    def __init__(self):
        self.name = 'CustAST_ASTall'

    def modify(self, idf, value):
        param_accis.modify_CustAST_ASTall(idf, value)

class CustAST_m:
    def __init__(self):
        self.name = 'CustAST_m'

    def modify(self, idf, value):
        param_accis.modify_CustAST_m(idf, value)

class CustAST_n:
    def __init__(self):
        self.name = 'CustAST_n'

    def modify(self, idf, value):
        param_accis.modify_CustAST_n(idf, value)

class CustAST_ACSToffset:
    def __init__(self):
        self.name = 'CustAST_ACSToffset'

    def modify(self, idf, value):
        param_accis.modify_CustAST_ACSToffset(idf, value)

class CustAST_AHSToffset:
    def __init__(self):
        self.name = 'CustAST_AHSToffset'

    def modify(self, idf, value):
        param_accis.modify_CustAST_AHSToffset(idf, value)

class CustAST_ASToffset:
    def __init__(self):
        self.name = 'CustAST_ASToffset'

    def modify(self, idf, value):
        param_accis.modify_CustAST_ASToffset(idf, value)

class CAT:
    def __init__(self):
        self.name = 'CAT'

    def modify(self, idf, value):
        param_accis.modify_CAT(idf, value)

class CATcoolOffset:
    def __init__(self):
        self.name = 'CATcoolOffset'

    def modify(self, idf, value):
        param_accis.modify_CATcoolOffset(idf, value)

class CATheatOffset:
    def __init__(self):
        self.name = 'CATheatOffset'

    def modify(self, idf, value):
        param_accis.modify_CATheatOffset(idf, value)

class ComfMod:
    def __init__(self):
        self.name = 'ComfMod'

    def modify(self, idf, value):
        param_accis.modify_ComfMod(idf, value)

class HVACmode:
    def __init__(self):
        self.name = 'HVACmode'

    def modify(self, idf, value):
        param_accis.modify_HVACmode(idf, value)

class VentCtrl:
    def __init__(self):
        self.name = 'VentCtrl'

    def modify(self, idf, value):
        param_accis.modify_VentCtrl(idf, value)

class VSToffset:
    def __init__(self):
        self.name = 'VSToffset'

    def modify(self, idf, value):
        param_accis.modify_VSToffset(idf, value)

class MinOToffset:
    def __init__(self):
        self.name = 'MinOToffset'

    def modify(self, idf, value):
        param_accis.modify_MinOToffset(idf, value)

class MaxWindSpeed:
    def __init__(self):
        self.name = 'MaxWindSpeed'

    def modify(self, idf, value):
        param_accis.modify_MaxWindSpeed(idf, value)

class ASTtol:
    def __init__(self):
        self.name = 'ASTtol'

    def modify(self, idf, value):
        param_accis.modify_ASTtol(idf, value)

class CoolSeasonStart:
    def __init__(self):
        self.name = 'CoolSeasonStart'

    def modify(self, idf, value):
        param_accis.modify_CoolSeasonStart(idf, value)

class CoolSeasonEnd:
    def __init__(self):
        self.name = 'CoolSeasonEnd'

    def modify(self, idf, value):
        param_accis.modify_CoolSeasonEnd(idf, value)

class SetpointAcc:
    def __init__(self):
        self.name = 'SetpointAcc'

    def modify(self, idf, value):
        param_accis.modify_SetpointAcc(idf, value)

class MaxTempDiffVOF:
    def __init__(self):
        self.name = 'MaxTempDiffVOF'

    def modify(self, idf, value):
        param_accis.modify_MaxTempDiffVOF(idf, value)

class MinTempDiffVOF:
    def __init__(self):
        self.name = 'MinTempDiffVOF'

    def modify(self, idf, value):
        param_accis.modify_MinTempDiffVOF(idf, value)

class MultiplierVOF:
    def __init__(self):
        self.name = 'MultiplierVOF'

    def modify(self, idf, value):
        param_accis.modify_MultiplierVOF(idf, value)




import pandas as pd
from besos import eppy_funcs as ef
from besos.evaluator import EvaluatorEP
from besos.parameters import Parameter, GenericSelector
from besos.problem import EPProblem
from besos.objectives import MeterReader

import accim.sim.accis_single_idf_funcs as accis
import accim.parametric_and_optimisation.funcs_for_besos.param_accis as bf

building = ef.get_building('TestModel_onlyGeometryForVRFsystem_2zones_CalcVent_V2310.idf')

accis.addAccis(
    idf=building,
    ScriptType='vrf_mm',
    SupplyAirTempInputMethod='temperature difference',
    Output_keep_existing=False,
    Output_type='standard',
    Output_freqs=['hourly'],
    # EnergyPlus_version='9.4',
    TempCtrl='temperature',
    # Output_gen_dataframe=True,
)

# gv = [i for i in building.idfobjects['EnergyManagementSystem:GlobalVariable']]

building.newidfobject(
    key='OUTPUT:METER',
    Key_Name='Electricity:Facility',
    Reporting_Frequency='RunPeriod'
)

building.newidfobject(
    key='OUTPUT:METER',
    Key_Name='Electricity:HVAC',
    Reporting_Frequency='RunPeriod'
)

# available_outputs = print_available_outputs_mod(building)

comfstand_range = [1, 2, 3]
cat_range = [1, 2, 3]
comfmod_range = [3]

samples = pd.DataFrame(columns=['ComfStand', 'CAT', 'ComfMod'])

index_val = 0
for i in comfstand_range:
    # samples.loc[index_val, 'ComfStand'] = i
    for j in cat_range:
        # samples.loc[index_val, 'CAT'] = j
        for k in comfmod_range:
            samples.loc[index_val, 'ComfMod'] = k
            samples.loc[index_val, 'ComfStand'] = i
            samples.loc[index_val, 'CAT'] = j

            index_val = index_val + 1

samples_filtered = bf.drop_invalid_param_combinations(samples)

##

parameters = [
    Parameter(
        name='ComfStand',
        # selector=GenericSelector(set=change_adaptive_coeff),
        selector=GenericSelector(set=bf.modify_ComfStand),
        # value_descriptors=CategoryParameter(options=comfstand_range)
    ),
    Parameter(
        name='CAT',
        # selector=GenericSelector(set=change_PMV_setpoints),
        selector=GenericSelector(set=bf.modify_CAT),
        # value_descriptors=CategoryParameter(options=cat_range)
    ),
    Parameter(
        name='ComfMod',
        # selector=GenericSelector(set=change_PMV_setpoints),
        selector=GenericSelector(set=bf.modify_ComfMod),
        # value_descriptors=CategoryParameter(options=comfmod_range)
    ),

]

# Objectives

# objs_comfhours = []
# for i in range(len(available_outputs.variablereaderlist)):
#     if 'hour' in available_outputs.variablereaderlist[i][1].lower():
#         objs_comfhours.append(
#             VariableReader(
#                 key_value=available_outputs.variablereaderlist[i][0],
#                 variable_name=available_outputs.variablereaderlist[i][1],
#                 frequency=available_outputs.variablereaderlist[i][2],
#                 name=available_outputs.variablereaderlist[i][1]
#             )
#         )

#

objectives = [
    MeterReader("Electricity:Facility", name="Total Electricity Usage"),
    MeterReader("Electricity:HVAC", name="HVAC Electricity Usage"),

    # VariableReader(
    #     key_value='Whole Building',
    #     variable_name='Facility Total HVAC Electricity Demand Rate',
    #     frequency='Hourly',
    #     name='HVAC Electricity usage'
    # )
]

problem = EPProblem(
    inputs=parameters,
    # outputs=objectives+objs_comfhours
    outputs=objectives
)


evaluator = EvaluatorEP(
    problem=problem,
    building=building,
    out_dir='outdir'
)

outputs = evaluator.df_apply(
    samples_filtered,
    keep_input=True,
    keep_dirs=True,
    # out_dir='outdir',
    processes=5
)

# outputs_mod = outputs
# outputs_mod['energy ratio'] = outputs_mod['HVAC Electricity Usage'] / outputs_mod['Total Electricity Usage']


# generated_buildings = [evaluator.generate_building(df=samples_short, index=i, file_name=f'short_sample_row_{i}') for i in range(5)]
evaluator.generate_building(df=samples, index=0, file_name='num_0')
evaluator.generate_building(df=samples, index=1, file_name='num_1')
evaluator.generate_building(df=samples, index=2, file_name='num_2')
# evaluator.generate_building(df=samples_short, index=3, file_name='num_3')
# evaluator.generate_building(df=samples_short, index=4, file_name='num_4')



# accim - Adaptive-Comfort-Control-Implemented Model
# Copyright (C) 2021-2025 Daniel Sánchez-García

# accim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# accim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

def scan_zones(self):
    """
    Used to scan the occupied zones in an idf.

    """
    import eppy
    self.occupiedZones_orig = []

    # Check if model comes from OpenStudio

    # Check if ZoneList or SpaceList are used
    occupiedZones_orig_osm = []

    self.spacelist_use = False
    try:
        if len(self.idf1.idfobjects['SPACELIST']) > 0:
            self.spacelist_use = True
            self.spacenames_for_ems_uniquekey = []
            self.spacenames_for_ems_name = []
            self.spacenames_for_ems_uniquekey_people = []
            self.zonenames_for_ems_with_sl = []
            for people in self.idf1.idfobjects['PEOPLE']:
                for spacelist in [i for i in self.idf1.idfobjects['SPACELIST'] if
                                  i.Name == people.Zone_or_ZoneList_or_Space_or_SpaceList_Name]:
                    for space in [i for i in self.idf1.idfobjects['SPACE'] if i.Space_Type == spacelist.Name]:
                        self.spacenames_for_ems_uniquekey.append(f'{space.Name} {spacelist.Name}')
                        self.spacenames_for_ems_name.append(space.Name)
                        self.spacenames_for_ems_uniquekey_people.append(f'{space.Name} {people.Name}')
                        occupiedZones_orig_osm.append(space.Zone_Name)
                        for zone in [i for i in self.idf1.idfobjects['ZONE'] if space.Zone_Name == i.Name]:
                            self.zonenames_for_ems_with_sl.append(zone.Name)
            self.spacenames_for_ems_uniquekey = list(set(self.spacenames_for_ems_uniquekey))
            self.spacenames_for_ems_name = list(set(self.spacenames_for_ems_name))
            self.spacenames_for_ems_uniquekey_people = list(set(self.spacenames_for_ems_uniquekey_people))
    except KeyError:
        idd = '-'.join([str(i) for i in self.idf1.idd_version])
        print('Searching Spacelist objects returned KeyError. '
              f'That means these are not supported in the EnergyPlus version {idd}')

    # occupiedZones_orig_osm = []
    # if len([h for h in self.idf1.idfobjects['zonelist']]) > 0:
    #     if len(self.idf1.idfobjects['zone']) == 1:
    #         no_of_zones = range(1, 2)
    #     else:
    #         no_of_zones = range(1, len(self.idf1.idfobjects['zone']))
    #
    #     for i in no_of_zones:
    #         for j in self.idf1.idfobjects['zonelist']:
    #             for k in self.idf1.idfobjects['zone']:
    #                 if k.Name in j[f'Zone_{i}_Name']:
    #                     occupiedZones_orig_osm.append(k.Name)
    #
    #     # for i in self.idf1.idfobjects['zone']:
    #     #     if all(i.Name not in [j for j in occupiedZones_orig_osm]):
    #     #         occupiedZones_orig_osm.append(i.Name)

    occupiedZones_orig_dsb = []
    for i in self.idf1.idfobjects['ZONE']:
        for k in self.idf1.idfobjects['PEOPLE']:
            try:
                if i.Name == k.Zone_or_ZoneList_or_Space_or_SpaceList_Name:
                    occupiedZones_orig_dsb.append(i.Name.upper())
            except eppy.bunch_subclass.BadEPFieldError:
                if i.Name == k.Zone_or_ZoneList_Name:
                    occupiedZones_orig_dsb.append(i.Name.upper())

    if self.spacelist_use:
        self.occupiedZones_orig = occupiedZones_orig_osm
        self.occupiedZones = [i.replace(' ', '_') for i in self.occupiedZones_orig]
        self.origin_dsb = False
        self.ems_objs_name = self.spacenames_for_ems_name
        self.ems_objs_key = self.spacenames_for_ems_uniquekey
        self.ems_zonenames = self.zonenames_for_ems_with_sl
        self.ems_zonenames_underscore = [i.replace(' ', '_') for i in self.ems_zonenames]
    else:
        self.occupiedZones_orig = occupiedZones_orig_dsb
        self.occupiedZones = [i.replace(':', '_') for i in self.occupiedZones_orig]
        self.origin_dsb = True
        self.ems_objs_name = self.occupiedZones
        self.ems_objs_key = self.occupiedZones_orig
        self.ems_zonenames = self.occupiedZones_orig
        self.ems_zonenames_underscore = self.occupiedZones_orig


def add_vrf_system(
        z,
        TempCtrl,
        verboseMode,
        EnergyPlus_version,
        SupplyAirTempInputMethod,
        eer,
        cop,
        VRFschedule
):
    if TempCtrl.lower() == 'temperature' or TempCtrl.lower() == 'temp':
        z.addOpTempTherm(verboseMode=verboseMode)
    elif TempCtrl.lower() == 'pmv':
        z.setPMVsetpoint(verboseMode=verboseMode)
    z.addBaseSchedules(verboseMode=verboseMode)
    z.setAvailSchOn(verboseMode=verboseMode)
    z.addVRFsystemSch(verboseMode=verboseMode)
    z.addCurveObj(verboseMode=verboseMode)
    z.addDetHVACobj(
        EnergyPlus_version=EnergyPlus_version,
        verboseMode=verboseMode,
        SupplyAirTempInputMethod=SupplyAirTempInputMethod,
        eer=eer,
        cop=cop,
        VRFschedule=VRFschedule
    )
